struct Allocations {
render_tasks: AllocationManager<RenderTaskType>,
paths: AllocationManager<PathEl>,
strips: AllocationManager<Strip>,
recorded_commands: AllocationManager<RecordedCommand>,
}Expand description
A structure to keep track of allocations that will be done while rendering with multi-threading.
Fields
render_tasks: AllocationManager<RenderTaskType>The render tasks of a batch. They will be filled by the main thread as new fill/stroke commands come in and consumed by worker threads as they process them.
paths: AllocationManager<PathEl>The path store of a batch. It will be filled by the main thread as new commands come in and be used by the worker thread to generate the strips of a path.
strips: AllocationManager<Strip>Stores allocations that are used by the worker thread to produce strips. They will be sent back to the main thread which then records the corresponding commands.
recorded_commands: AllocationManager<RecordedCommand>The commands produced by a worker thread, which will be recorded by the main thread.
Implementations
Sourceimpl Allocations
impl Allocations
Sourcefn get(&mut self) -> AllocationGroup
fn get(&mut self) -> AllocationGroup
Return a new allocation group.
The group is guaranteed to have been cleared.
fn put(&mut self, allocation: AllocationGroup)