pub(crate) struct OwnedClip {
strips: Box<[Strip]>,
alphas: Box<[u8]>,
bbox: RectU16,
}Fields
strips: Box<[Strip]>alphas: Box<[u8]>bbox: RectU16A coarse bounding box of the clip path in pixel coordinates.
These bounds have already been intersected with the viewport.